/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the License).  You may not use this file except in
 * compliance with the License.
 *
 * You can obtain a copy of the license at
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing
 * permissions and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL
 * Header Notice in each file and include the License file
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.
 * If applicable, add the following below the CDDL Header,
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information:
 * "Portions Copyrighted [year] [contextPath of copyright owner]"
 *
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

package com.sun.grizzly.comet;

import com.sun.grizzly.Pipeline;
import com.sun.grizzly.PipelineFullException;
import com.sun.grizzly.http.TaskBase;
import java.io.IOException;
import java.nio.channels.SelectionKey;
import java.util.Iterator;
import java.util.ArrayList;

/**
 * Default Notificationhandler that uses the same a Grizzly Pipeline
 * to execute the notification process.
 *
 * @author Jeanfrancois Arcand
 */
public class DefaultNotificationHandler implements NotificationHandler{
    
    /**
     * The <code>Pipeline</code> used to execute threaded notification.
     */
    protected Pipeline pipeline;

    
    /**
     * <tt>true</tt> if the caller of CometContext.notify should block when 
     * notifying other CometHandler.
     */
    protected boolean blockingNotification = false;
    

    /**
     * Set the <code>Pipeline</code> used for notifying the CometHandler.
     */
    protected void setPipeline(Pipeline pipeline){
        this.pipeline = pipeline;
    }

    
    /**
     * Return <tt>true</tt> if the invoker of notify() should block when
     * notifying Comet Handlers.
     */
    public boolean isBlockingNotification() {
        return blockingNotification;
    }

    
    /**
     * Set to <tt>true</tt> if the invoker of notify() should block when
     * notifying Comet Handlers.
     */
    public void setBlockingNotification(boolean blockingNotification) {
        this.blockingNotification = blockingNotification;
    }

        
    /**
     * Notify all <code>CometHandler</code>. 
     * @param cometEvent the CometEvent used to notify CometHandler
     * @param iteratorHandlers An iterator over a list of CometHandler
     */
    public void notify(final CometEvent cometEvent,final Iterator<CometHandler> 
            iteratorHandlers) throws IOException{
        if (blockingNotification || pipeline == null){
            notify0(cometEvent,iteratorHandlers);
        } else {
            try{
                pipeline.execute(new TaskBase(){
                    public void doTask() throws IOException{
                        notify0(cometEvent,iteratorHandlers);
                    }
                });
            } catch (PipelineFullException ex){
                throw new IOException(ex.getMessage());
            }                
        }
    }


    protected void notify0(CometEvent cometEvent,Iterator<CometHandler> iteratorHandlers) 
            throws IOException{
        ArrayList<Throwable> exceptions = null;
        while(iteratorHandlers.hasNext()){
            try{
                notify0(cometEvent,iteratorHandlers.next());
            } catch (Throwable ex){
                ex.printStackTrace();
                if (exceptions == null){
                    exceptions = new ArrayList<Throwable>();
                }
                exceptions.add(ex);
            }
        }
        if (exceptions != null){
            StringBuffer errorMsg = new StringBuffer();
            for(Throwable t: exceptions){
                errorMsg.append(t.getMessage());
            }
            throw new IOException(errorMsg.toString());
        }
    }
    

    /**
     * Notify a single <code>CometHandler</code>. 
     * @param cometEvent the CometEvent used to notify CometHandler
     * @param cometHandler a CometHandler
     */
    public void notify(final CometEvent cometEvent,final CometHandler cometHandler) 
            throws IOException{
        if (blockingNotification || pipeline == null){
            notify0(cometEvent,cometHandler);
        } else {
            final ArrayList<Throwable> exceptions 
                    = new ArrayList<Throwable>();
            try{
                pipeline.execute(new TaskBase(){
                    public void doTask() throws IOException{
                        try{
                            notify0(cometEvent,cometHandler);
                        } catch (Throwable ex){
                            exceptions.add(ex);
                        }
                        if (exceptions.size() > 0){
                            StringBuffer errorMsg = new StringBuffer();
                            for(Throwable t: exceptions){
                                errorMsg.append(t.getMessage());
                            }
                            throw new IOException(errorMsg.toString());
                        }
                    }
                });
            } catch (PipelineFullException ex){
                throw new IOException(ex.getMessage());
            }
        }
    }

    
    /**
     * Notify a <code>CometHandler</code>.
     * 
     * CometEvent.INTERRUPT -> <code>CometHandler.onInterrupt</code>
     * CometEvent.NOTIFY -> <code>CometHandler.onEvent</code>
     * CometEvent.INITIALIZE -> <code>CometHandler.onInitialize</code>
     * CometEvent.TERMINATE -> <code>CometHandler.onTerminate</code>
     * CometEvent.READ -> <code>CometHandler.onEvent</code>
     *
     * @param attachment An object shared amongst <code>CometHandler</code>. 
     * @param cometHandler The CometHandler to invoke. 
     */
    protected void notify0(CometEvent cometEvent,CometHandler cometHandler) 
            throws IOException{
        switch (cometEvent.getType()) {
            case CometEvent.INTERRUPT:
                cometHandler.onInterrupt(cometEvent);
                break;
            case CometEvent.NOTIFY:
                cometHandler.onEvent(cometEvent);
                break;
            case CometEvent.READ:
                cometHandler.onEvent(cometEvent);
                break;                    
            case CometEvent.INITIALIZE:
                cometHandler.onInitialize(cometEvent);
                break;      
            case CometEvent.TERMINATE:
                cometHandler.onTerminate(cometEvent);
                break;                       
            default:
                cometHandler.onEvent(cometEvent);
                break;
        }
    }
}
